#!/usr/bin/env python3

import os
import sys
import json
from http.server import BaseHTTPRequestHandler, HTTPServer
from threading import Event, Thread

LISTENER_ADDRESS = "127.0.0.1"
LISTENER_PORT = 8090

def start_http_listener(queue):
    def request_handler(*args):
        TelemetryRequestHandler(queue, *args)
    http_server = HTTPServer((LISTENER_ADDRESS, LISTENER_PORT), request_handler)
    started_event = Event()
    server_thread = Thread(target=run_server, daemon=True, args=(started_event, http_server, ))
    server_thread.start()
    is_event_started = started_event.wait(timeout = 9)
    if not is_event_started:
        raise Exception("server_thread has timedout before starting")
    listener_url = "http://{0}:{1}".format(LISTENER_ADDRESS,LISTENER_PORT)
    return listener_url

def run_server(started_event, http_server):
    started_event.set()
    try:
        http_server.serve_forever()
    except Exception as e:
        raise Exception("Error in HTTP server", e)
    finally:
        if http_server:
            http_server.shutdown()

class TelemetryRequestHandler(BaseHTTPRequestHandler):
    def __init__(self, queue, *args):
        self.queue = queue
        BaseHTTPRequestHandler.__init__(self, *args)

    def do_POST(self):
        try:
            cl = self.headers.get("Content-Length")
            if cl:
                data_len = int(cl)
            else:
                data_len = 0
            content = self.rfile.read(data_len)
            self.send_response(200)
            self.end_headers()
            print(content.decode("utf-8"), flush=True)
            batch = json.loads(content.decode("utf-8"))
            self.queue.put(batch)
        except Exception as e:
            raise Exception("Error processing message", e)


